#ifndef PAPIAPI_H_INCLUDED
#define PAPIAPI_H_INCLUDED

/**********************************************************************
 *
 * Open Standard Print API (PAPI) - High-level Functions
 *
 * Copyright (c) 2002 International Business Machines Corp.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 *********************************************************************/

#include <ippclient.h>
#include <time.h>

/*---------------------------------------------------------------------
 * Constants and Type definitions
 *-------------------------------------------------------------------*/

#define PAPI_MAX_NAME IPP_MAX_NAME

/* papi_status_t:  Describes the success/failure of an operation */
#define papi_status_t ipp_status_t
#define PAPI_OK                             IPP_OK
#define PAPI_OK_SUBST                       IPP_OK_SUBST
#define PAPI_OK_CONFLICT                    IPP_OK_CONFLICT
#define PAPI_OK_IGNORED_SUBSCRIPTIONS       IPP_OK_IGNORED_SUBSCRIPTIONS
#define PAPI_OK_IGNORED_NOTIFICATIONS       IPP_OK_IGNORED_NOTIFICATIONS
#define PAPI_OK_TOO_MANY_EVENTS             IPP_OK_TOO_MANY_EVENTS
#define PAPI_OK_BUT_CANCEL_SUBSCRIPTION     IPP_OK_BUT_CANCEL_SUBSCRIPTION
#define PAPI_REDIRECTION_OTHER_SITE         IPP_REDIRECTION_OTHER_SITE
#define PAPI_BAD_REQUEST                    IPP_BAD_REQUEST
#define PAPI_FORBIDDEN                      IPP_FORBIDDEN
#define PAPI_NOT_AUTHENTICATED              IPP_NOT_AUTHENTICATED
#define PAPI_NOT_AUTHORIZED                 IPP_NOT_AUTHORIZED
#define PAPI_NOT_POSSIBLE                   IPP_NOT_POSSIBLE
#define PAPI_TIMEOUT                        IPP_TIMEOUT
#define PAPI_NOT_FOUND                      IPP_NOT_FOUND
#define PAPI_GONE                           IPP_GONE
#define PAPI_REQUEST_ENTITY                 IPP_REQUEST_ENTITY
#define PAPI_REQUEST_VALUE                  IPP_REQUEST_VALUE
#define PAPI_DOCUMENT_FORMAT                IPP_DOCUMENT_FORMAT
#define PAPI_ATTRIBUTES                     IPP_ATTRIBUTES
#define PAPI_URI_SCHEME                     IPP_URI_SCHEME
#define PAPI_CHARSET                        IPP_CHARSET
#define PAPI_CONFLICT                       IPP_CONFLICT
#define PAPI_COMPRESSION_NOT_SUPPORTED      IPP_COMPRESSION_NOT_SUPPORTED
#define PAPI_COMPRESSION_ERROR              IPP_COMPRESSION_ERROR
#define PAPI_DOCUMENT_FORMAT_ERROR          IPP_DOCUMENT_FORMAT_ERROR
#define PAPI_DOCUMENT_ACCESS_ERROR          IPP_DOCUMENT_ACCESS_ERROR
#define PAPI_ATTRIBUTES_NOT_SETTABLE        IPP_ATTRIBUTES_NOT_SETTABLE
#define PAPI_IGNORED_ALL_SUBSCRIPTIONS      IPP_IGNORED_ALL_SUBSCRIPTIONS
#define PAPI_TOO_MANY_SUBSCRIPTIONS         IPP_TOO_MANY_SUBSCRIPTIONS
#define PAPI_IGNORED_ALL_NOTIFICATIONS      IPP_IGNORED_ALL_NOTIFICATIONS
#define PAPI_PRINT_SUPPORT_FILE_NOT_FOUND   IPP_PRINT_SUPPORT_FILE_NOT_FOUND
#define PAPI_INTERNAL_ERROR                 IPP_INTERNAL_ERROR
#define PAPI_OPERATION_NOT_SUPPORTED        IPP_OPERATION_NOT_SUPPORTED
#define PAPI_SERVICE_UNAVAILABLE            IPP_SERVICE_UNAVAILABLE
#define PAPI_VERSION_NOT_SUPPORTED          IPP_VERSION_NOT_SUPPORTED
#define PAPI_DEVICE_ERROR                   IPP_DEVICE_ERROR
#define PAPI_TEMPORARY_ERROR                IPP_TEMPORARY_ERROR
#define PAPI_NOT_ACCEPTING                  IPP_NOT_ACCEPTING
#define PAPI_PRINTER_BUSY                   IPP_PRINTER_BUSY
#define PAPI_ERROR_JOB_CANCELLED            IPP_ERROR_JOB_CANCELLED
#define PAPI_MULTIPLE_JOBS_NOT_SUPPORTED    IPP_MULTIPLE_JOBS_NOT_SUPPORTED
#define PAPI_PRINTER_IS_DEACTIVATED         IPP_PRINTER_IS_DEACTIVATED
typedef enum
{
	PAPI_JOB_TICKET_NOT_SUPPORTED = 0x0700
}
papi_status_values_t;

/* papi_service_t:  PAPI service handle */
typedef void* papi_service_t;

/* papi_encryption_t:  Encryption type */
typedef enum
{
  PAPI_ENCRYPT_IF_REQUESTED,/* Encrypt if requested (TLS upgrade) */
  PAPI_ENCRYPT_NEVER,    /* Never encrypt */
  PAPI_ENCRYPT_REQUIRED, /* Encryption is required (TLS upgrade) */
  PAPI_ENCRYPT_ALWAYS    /* Always encrypt (SSL) */
} papi_encryption_t;

/* papi_attribute_value_type_t:  Attribute Type */
typedef enum
{
        PAPI_STRING,
        PAPI_INTEGER,
        PAPI_BOOLEAN,
        PAPI_RANGE,
        PAPI_RESOLUTION,
        PAPI_DATETIME,
        PAPI_COLLECTION
} papi_attribute_value_type_t;

/* Boolean values */
enum
{
	PAPI_FALSE = 0,
	PAPI_TRUE  = 1
};

/* papi_res_t:  Resolution units */
typedef enum			
{
  PAPI_RES_PER_INCH = 3,
  PAPI_RES_PER_CM
} papi_res_t;

struct papi_attribute_str;

/* papi_attribute_value_t:  Attribute Value */
typedef union
{
    char* string;     /* PAPI_STRING value */

    int   integer;    /* PAPI_INTEGER value */

    char  boolean;    /* PAPI_BOOLEAN value */

    struct            /* PAPI_RANGE value */
    {
        int lower;
        int upper;
    } range;

    struct            /* PAPI_RESOLUTION value */
    {
        int xres;
        int yres;
        papi_res_t units;
    } resolution;

    time_t datetime;  /* PAPI_DATETIME value */

    struct papi_attribute_str**
        collection;   /* PAPI_COLLECTION value */
} papi_attribute_value_t;

/* papi_attribute_t:  Attribute and Values */
typedef struct papi_attribute_str
{
    char* name;                       /* attribute name */
    papi_attribute_value_type_t type; /* type of values */
    papi_attribute_value_t** values;  /* list of values */
} papi_attribute_t;

/* Attribute update types */
#define PAPI_ATTR_APPEND  0x0001 /* Add values to attr  */
#define PAPI_ATTR_REPLACE 0x0002 /* Delete existing
                               values then add new ones */
#define PAPI_ATTR_EXCL    0x0004 /* Fail if attr exists */

/* papi_jt_format_t:  Job Ticket Format */
typedef enum
{
  PAPI_JT_FORMAT_JDF = 0,         /* Job Definition Format */
  PAPI_JT_FORMAT_PWG = 1          /* PWG Job Ticket Format */
} papi_jt_format_t;

/* papi_job_ticket_t:  Job Ticket */
typedef struct papi_job_ticket_s
{
    papi_jt_format_t format;      /* Format of job ticket */
    char*            ticket_data; /* Buffer containing the job
                                     ticket data.  If NULL,
                                     file_name must be specified */
    char*            file_name;   /* Name of the file containing
                                     the job ticket data. If
                                     ticket_data is specified, then
                                     file_name is ignored. */
} papi_job_ticket_t;

/* papi_job_t:  Job object */
typedef void* papi_job_t;

/* papi_stream_t:  Stream object */
typedef void* papi_stream_t;

/* papi_printer_t:  Printer object */
typedef void* papi_printer_t;

/* papi_filter_type_t:  Type of list filter */
typedef enum
{
    PAPI_FILTER_BITMASK = 0
    /* future filter types may be added here */
} papi_filter_type_t;

/* papi_filter_t:  List filter */
typedef struct
{
    papi_filter_type_t   type; /* Type of filter specified */

    union
    {
    	/* Bitmask filter */
    	struct
    	{
	        unsigned int  mask;  /* bit mask */
	        unsigned int  value; /* bit value */
		} bitmask;

        /* future filter types may be added here */
    } filter;
} papi_filter_t;

/* Finishings enum values */
typedef enum			
{
  PAPI_FINISHINGS_NONE = 3,
  PAPI_FINISHINGS_STAPLE,
  PAPI_FINISHINGS_PUNCH,
  PAPI_FINISHINGS_COVER,
  PAPI_FINISHINGS_BIND,
  PAPI_FINISHINGS_SADDLE_STITCH,
  PAPI_FINISHINGS_EDGE_STITCH,
  PAPI_FINISHINGS_FOLD,
  PAPI_FINISHINGS_TRIM,
  PAPI_FINISHINGS_BALE,
  PAPI_FINISHINGS_BOOKLET_MAKER,
  PAPI_FINISHINGS_JOB_OFFSET,
  PAPI_FINISHINGS_STAPLE_TOP_LEFT = 20,
  PAPI_FINISHINGS_STAPLE_BOTTOM_LEFT,
  PAPI_FINISHINGS_STAPLE_TOP_RIGHT,
  PAPI_FINISHINGS_STAPLE_BOTTOM_RIGHT,
  PAPI_FINISHINGS_EDGE_STITCH_LEFT,
  PAPI_FINISHINGS_EDGE_STITCH_TOP,
  PAPI_FINISHINGS_EDGE_STITCH_RIGHT,
  PAPI_FINISHINGS_EDGE_STITCH_BOTTOM,
  PAPI_FINISHINGS_STAPLE_DUAL_LEFT,
  PAPI_FINISHINGS_STAPLE_DUAL_TOP,
  PAPI_FINISHINGS_STAPLE_DUAL_RIGHT,
  PAPI_FINISHINGS_STAPLE_DUAL_BOTTOM,
  PAPI_FINISHINGS_BIND_LEFT = 50,
  PAPI_FINISHINGS_BIND_TOP,
  PAPI_FINISHINGS_BIND_RIGHT,
  PAPI_FINISHINGS_BIND_BOTTOM
} papi_finish_t;

/* Orientation enum values */
typedef enum			
{
  PAPI_PORTRAIT = 3,		/* No rotation */
  PAPI_LANDSCAPE,		/* 90 degrees counter-clockwise */
  PAPI_REVERSE_LANDSCAPE,	/* 90 degrees clockwise */
  PAPI_REVERSE_PORTRAIT		/* 180 degrees */
} papi_orient_t;

/* Print quality enum values */
typedef enum			
{
  PAPI_QUALITY_DRAFT = 3,
  PAPI_QUALITY_NORMAL,
  PAPI_QUALITY_HIGH
} papi_quality_t;

/* Job state enum values */
typedef enum			
{
  PAPI_JOB_PENDING = 3,
  PAPI_JOB_HELD,
  PAPI_JOB_PROCESSING,
  PAPI_JOB_STOPPED,
  PAPI_JOB_CANCELLED,
  PAPI_JOB_ABORTED,
  PAPI_JOB_COMPLETED
} papi_jstate_t;

/* Printer state enum values */
typedef enum			
{
  PAPI_PRINTER_IDLE = 3,
  PAPI_PRINTER_PROCESSING,
  PAPI_PRINTER_STOPPED
} papi_pstate_t;

/* Printer list filter mask values */
enum
{
  PAPI_PRINTER_LOCAL = 0x0000,          /* Local printer or class */
  PAPI_PRINTER_CLASS = 0x0001,          /* Printer class */
  PAPI_PRINTER_REMOTE = 0x0002,         /* Remote printer or class */
  PAPI_PRINTER_BW = 0x0004,                 /* Can do B&amp;W printing */
  PAPI_PRINTER_COLOR = 0x0008,          /* Can do color printing */
  PAPI_PRINTER_DUPLEX = 0x0010,         /* Can do duplexing */
  PAPI_PRINTER_STAPLE = 0x0020,         /* Can staple output */
  PAPI_PRINTER_COPIES = 0x0040,         /* Can do copies */
  PAPI_PRINTER_COLLATE = 0x0080,        /* Can collage copies */
  PAPI_PRINTER_PUNCH = 0x0100,          /* Can punch output */
  PAPI_PRINTER_COVER = 0x0200,          /* Can cover output */
  PAPI_PRINTER_BIND = 0x0400,           /* Can bind output */
  PAPI_PRINTER_SORT = 0x0800,           /* Can sort output */
  PAPI_PRINTER_SMALL = 0x1000,          /* Can do Letter/Legal/A4 */
  PAPI_PRINTER_MEDIUM = 0x2000,         /* Can do Tabloid/B/C/A3/A2 */
  PAPI_PRINTER_LARGE = 0x4000,          /* Can do D/E/A1/A0 */
  PAPI_PRINTER_VARIABLE = 0x8000,       /* Can do variable sizes */
  PAPI_PRINTER_IMPLICIT = 0x10000,      /* Implicit class */
  PAPI_PRINTER_DEFAULT = 0x20000,       /* Default printer on network */
  PAPI_PRINTER_OPTIONS = 0xfffc         /* ~(CLASS | REMOTE | IMPLICIT) */
};

/* List jobs options */
#define PAPI_LIST_JOBS_OTHERS        0x0001 /* return jobs other than
                                               those submitted by the
                                               user name assoc with
                                               the handle */
#define PAPI_LIST_JOBS_COMPLETED     0x0002 /* return completed jobs */
#define PAPI_LIST_JOBS_NOT_COMPLETED 0x0004 /* return not-completed
                                               jobs */
#define PAPI_LIST_JOBS_ALL           0xFFFF /* return all jobs */

/**********************************************************************
 * Function prototypes
 *
 * See the PAPI documentation (papi.pdf) for descriptions and
 * examples of how to use these functions.
 *********************************************************************/
#  ifdef __cplusplus
extern "C" {
#  endif /* __cplusplus */

/*---------------------------------------------------------------------
 * Service API
 *-------------------------------------------------------------------*/
extern papi_status_t papiServiceCreate(
         papi_service_t*         handle,
         const char*             service_name,
         const char*             user_name,
         const char*             password,
         int (*authCB)(papi_service_t svc),
         const papi_encryption_t encryption,
         void*                   app_data );

extern void papiServiceDestroy(
         papi_service_t handle );

extern papi_status_t papiServiceSetUserName(
         papi_service_t handle,
         const char* user_name );

extern papi_status_t papiServiceSetPassword(
         papi_service_t handle,
         const char* password );

extern papi_status_t papiServiceSetEncryption(
         papi_service_t handle,
         const papi_encryption_t encryption );

extern papi_status_t papiServiceSetAuthCB(
         papi_service_t handle,
         const int (*authCB)(papi_service_t svc) );

extern papi_status_t papiServiceSetAppData(
         papi_service_t handle,
         const void*    app_data );

extern char* papiServiceGetServiceName(
         papi_service_t handle );

extern char* papiServiceGetUserName(
         papi_service_t handle );

extern char* papiServiceGetPassword(
         papi_service_t handle );

extern papi_encryption_t papiServiceGetEncryption(
         papi_service_t handle );

extern void* papiServiceGetAppData(
         papi_service_t handle );

extern const char* papiServiceGetStatusMessage(
         const papi_service_t handle );

/*---------------------------------------------------------------------
 * Printer API
 *-------------------------------------------------------------------*/
extern papi_status_t papiPrintersList(
                papi_service_t     handle,
          const char*              requested_attrs[],
          const papi_filter_t*     filter,
                papi_printer_t**   printers );

extern papi_status_t papiPrinterQuery(
                papi_service_t     handle,
          const char*              name,
          const char*              requested_attrs[],
          const papi_attribute_t** job_attrs,
                papi_printer_t*    printer );

extern papi_status_t papiPrinterModify(
                papi_service_t     handle,
          const char*               printer_name,
          const papi_attribute_t**  attrs,
                papi_printer_t*     printer );

extern papi_status_t papiPrinterPause(
                papi_service_t     handle,
          const char*              name );

extern papi_status_t papiPrinterResume(
                papi_service_t     handle,
          const char*              name );

extern papi_status_t papiPrinterPurgeJobs(
                papi_service_t     handle,
          const char*              name,
                papi_job_t**       result );

extern papi_status_t papiPrinterListJobs(
                papi_service_t     handle,
          const char*              printer,
          const char*              requested_attrs[],
          const int                type_mask,
          const int                max_num_jobs,
                papi_job_t**       jobs );

extern papi_attribute_t** papiPrinterGetAttributeList(
                papi_printer_t     printer );

extern void papiPrinterFree(
                papi_printer_t     printer );

extern void papiPrinterListFree(
                papi_printer_t*    printers );

/*---------------------------------------------------------------------
 * Attributes API
 *-------------------------------------------------------------------*/
extern papi_status_t papiAttributeListAdd(
          papi_attribute_t*** attrs,
          const int add_flags,
          const char* name,
          const papi_attribute_value_type_t type,
          const papi_attribute_value_t* value );

extern papi_status_t papiAttributeListAddString(
          papi_attribute_t*** attrs,
          const int add_flags,
          const char* name,
          const char* value);

extern papi_status_t papiAttributeListAddInteger(
          papi_attribute_t*** attrs,
          const int add_flags,
          const char* name,
          const int value);

extern papi_status_t papiAttributeListAddBoolean(
          papi_attribute_t*** attrs,
          const int add_flags,
          const char* name,
          const char value);

extern papi_status_t papiAttributeListAddRange(
          papi_attribute_t*** attrs,
          const int add_flags,
          const char* name,
          const int lower,
          const int upper );

extern papi_status_t papiAttributeListAddResolution(
          papi_attribute_t*** attrs,
          const int add_flags,
          const char* name,
          const int xres,
          const int yres,
          const papi_res_t units );

extern papi_status_t papiAttributeListAddDatetime(
          papi_attribute_t*** attrs,
          const int add_flags,
          const char* name,
          const time_t date_time );

extern papi_status_t papiAttributeListAddCollection(
          papi_attribute_t*** attrs,
          const int add_flags,
          const char* name,
          const papi_attribute_t** collection );

extern papi_status_t papiAttributeDelete(
          papi_attribute_t*** attrs,
          const char* name );

extern papi_status_t papiAttributeListGetValue(
          const papi_attribute_t** attrs,
                void** iterator,
          const char* name,
          const papi_attribute_value_type_t type,
                papi_attribute_value_t** value );

extern papi_status_t papiAttributeListGetString(
          const papi_attribute_t** attrs,
                void** iterator,
          const char* name,
                char** value );

extern papi_status_t papiAttributeListGetInteger(
          const papi_attribute_t** attrs,
                void** iterator,
          const char* name,
                int* value );

extern papi_status_t papiAttributeListGetBoolean(
          const papi_attribute_t** attrs,
                void** iterator,
          const char* name,
                char* value );

extern papi_status_t papiAttributeListGetRange(
          const papi_attribute_t** attrs,
                void** iterator,
          const char* name,
                int* lower,
                int* upper );

extern papi_status_t papiAttributeListGetResolution(
          const papi_attribute_t** attrs,
                void** iterator,
          const char* name,
                int* xres,
                int* yres,
                papi_res_t* units );

extern papi_status_t papiAttributeListGetDatetime(
          const papi_attribute_t** attrs,
                void** iterator,
          const char* name,
                time_t* date_time);

extern papi_status_t papiAttributeListGetCollection(
          const papi_attribute_t** attrs,
                void** iterator,
          const char* name,
                papi_attribute_t*** collection);

extern void papiAttributeListFree(
                papi_attribute_t** attrs );

extern papi_attribute_t* papiAttributeListFind(
          const papi_attribute_t** attrs,
          const char*              name );

extern papi_attribute_t* papiAttributeListGetNext(
          const papi_attribute_t** attrs,
                void**             iterator );

extern papi_status_t papiAttributeListFromString(
          const papi_attribute_t*** attrs,
          const int                add_flags,
          const char*              buffer );

extern papi_status_t papiAttributeListToString(
          const papi_attribute_t** attrs,
          const char*              attr_delim,
                char*              buffer,
          const size_t             buflen );


/*---------------------------------------------------------------------
 * Job API
 *-------------------------------------------------------------------*/
extern papi_status_t papiJobSubmit(
                papi_service_t       handle,
          const char*                printer_name,
          const papi_attribute_t**   job_attributes,
          const papi_job_ticket_t*   job_ticket,
          const char**               file_names,
                papi_job_t*          job );

extern papi_status_t papiJobSubmitByReference(
                papi_service_t       handle,
          const char*                printer_name,
          const papi_attribute_t**   job_attributes,
          const papi_job_ticket_t*   job_ticket,
          const char**               file_names,
                papi_job_t*          job );

extern papi_status_t papiJobValidate(
                papi_service_t       handle,
          const char*                printer_name,
          const papi_attribute_t**   job_attributes,
          const papi_job_ticket_t*   job_ticket,
          const char**               file_names,
                papi_job_t*          job );

extern papi_status_t papiJobStreamOpen(
                papi_service_t       handle,
          const char*                printer_name,
          const papi_attribute_t**   job_attributes,
          const papi_job_ticket_t*   job_ticket,
                papi_stream_t*       stream );

extern papi_status_t papiJobStreamWrite(
                papi_service_t       handle,
                papi_stream_t        stream,
          const char*                buffer,
          const size_t               buflen );

extern papi_status_t papiJobStreamClose(
                papi_service_t       handle,
                papi_stream_t        stream,
                papi_job_t*          job );

extern papi_status_t papiJobQuery(
                papi_service_t     handle,
          const char*              printer_name,
          const int32_t            job_id,
          const char*              requested_attrs[],
                papi_job_t*        job );

extern papi_status_t papiJobModify(
                papi_service_t     handle,
          const char*              printer_name,
          const int32_t            job_id,
          const papi_attribute_t** job_attrs,
                papi_job_t*        job );

extern papi_status_t papiJobCancel(
                papi_service_t     handle,
          const char*              printer_name,
          const int32_t            job_id );

extern papi_status_t papiJobHold(
                papi_service_t      handle,
          const char*               printer_name,
          const int32_t             job_id,
          const char*               hold_until,
          const time_t*             hold_until_time );

extern papi_status_t papiJobRelease(
                papi_service_t      handle,
          const char*               printer_name,
          const int32_t             job_id );

extern papi_status_t papiJobRestart(
                papi_service_t      handle,
          const char*               printer_name,
          const int32_t             job_id );

extern papi_attribute_t** papiJobGetAttributeList(
                papi_job_t      job );

extern char* papiJobGetPrinterName(
                papi_job_t      job );

extern int32_t papiJobGetId(
                papi_job_t      job );

extern papi_job_ticket_t* papiJobGetJobTicket(
                papi_job_t      job );

extern void papiJobFree(
                papi_job_t      job );

extern void papiJobListFree(
                papi_job_t*     jobs );

/*---------------------------------------------------------------------
 * Miscellaneous API
 *-------------------------------------------------------------------*/
extern char* papiStatusString(
          const papi_status_t status );

#  ifdef __cplusplus
}
#  endif /* __cplusplus */

#endif /* PAPIAPI_H_INCLUDED */
